@@ -20,12 +20,12 @@
 
 
 #include <string.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include "userprefs.h"
 #include <sys/stat.h>
+#include "logview.h"
 
 #define LOG_CANVAS_H             400
 #define LOG_CANVAS_W             600
@@ -35,8 +35,13 @@
 #define GCONF_LOGFILE "/apps/gnome-system-log/logfile"
 #define GCONF_LOGFILES "/apps/gnome-system-log/logfiles"
 #define GCONF_FONTSIZE_KEY "/apps/gnome-system-log/fontsize"
+#define GCONF_MONOSPACE_FONT_NAME "/desktop/gnome/interface/monospace_font_name"
+#define GCONF_MENUS_HAVE_TEAROFF "/desktop/gnome/interface/menus_have_tearoff"
 
-GSList*
+static GConfClient *client = NULL;
+static UserPrefs *prefs;
+
+static GSList*
 parse_syslog(gchar *syslog_file) {
 
 	/* Most of this stolen from sysklogd sources */
@@ -71,8 +76,8 @@ parse_syslog(gchar *syslog_file) {
 	return logfiles; 
 }
 
-void
-prefs_create_defaults (UserPrefsStruct *prefs)
+static void
+prefs_create_defaults (UserPrefs *p)
 {
 	int i;
 	gchar *logfiles[] = {"/var/adm/messages",
@@ -86,6 +91,8 @@ prefs_create_defaults (UserPrefsStruct *prefs)
 	GSList *logs = NULL;
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
+
+    g_assert (p != NULL);
 	
 	/* For first time running, try parsing various logfiles */
 	/* Try to parse syslog.conf to get logfile names */
@@ -102,43 +109,37 @@ prefs_create_defaults (UserPrefsStruct *prefs)
 	}
 
 	if (logs != NULL) {
-		prefs->logs = logs;
-		prefs->logfile = logs->data;
+		p->logs = logs;
+		p->logfile = logs->data;
 	}
 }
 
-void
-prefs_free_loglist (UserPrefsStruct *prefs)
-{
-	GSList *list;
-	g_slist_free (prefs->logs);
-	prefs->logs = NULL;
-}
-
-UserPrefsStruct *
+static UserPrefs *
 prefs_load (GConfClient *client)
 {
 	gchar *logfile = NULL;
 	int width, height, fontsize;
-	UserPrefsStruct *prefs;
+	UserPrefs *p;
 	GSList *list;
 	gboolean found;
 
-	prefs = g_new0 (UserPrefsStruct, 1);
+    g_assert (client != NULL);
 
-	prefs->logs = gconf_client_get_list (client, GCONF_LOGFILES, GCONF_VALUE_STRING, NULL);
-	if (prefs->logs == NULL)
-		prefs_create_defaults (prefs);
+	p = g_new0 (UserPrefs, 1);
+
+	p->logs = gconf_client_get_list (client, GCONF_LOGFILES, GCONF_VALUE_STRING, NULL);
+	if (p->logs == NULL)
+		prefs_create_defaults (p);
 
 	logfile = gconf_client_get_string (client, GCONF_LOGFILE, NULL);
 	if (logfile != NULL && strcmp (logfile, "") && file_is_log(logfile, FALSE)) {
-		prefs->logfile = g_strdup (logfile);
+		p->logfile = g_strdup (logfile);
 		g_free (logfile);
 	}
 
 	found = FALSE;
-	for (list = prefs->logs; list!=NULL; list = g_slist_next (list)) {
-		if (g_ascii_strncasecmp (list->data, prefs->logfile, 255) == 0)
+	for (list = p->logs; list!=NULL; list = g_slist_next (list)) {
+		if (g_ascii_strncasecmp (list->data, p->logfile, 255) == 0)
 			found = TRUE;
 	}
 
@@ -146,17 +147,97 @@ prefs_load (GConfClient *client)
 	height = gconf_client_get_int (client, GCONF_HEIGHT_KEY, NULL);
 	fontsize = gconf_client_get_int (client, GCONF_FONTSIZE_KEY, NULL);
 
-	prefs->width = (width == 0 ? LOG_CANVAS_W : width);
-	prefs->height = (height == 0 ? LOG_CANVAS_H : height);
-	prefs->fontsize = fontsize;
+	p->width = (width == 0 ? LOG_CANVAS_W : width);
+	p->height = (height == 0 ? LOG_CANVAS_H : height);
+	p->fontsize = fontsize;
 
-	return prefs;
+	return p;
+}
+
+static void
+prefs_monospace_font_changed (GConfClient *client, guint id, 
+                                GConfEntry *entry, gpointer data)
+{
+    GtkWidget *view = data;
+    gchar *monospace_font_name;
+    PangoFontDescription *fontdesc;
+
+    g_assert (client != NULL);
+    
+    monospace_font_name = gconf_client_get_string (client, GCONF_MONOSPACE_FONT_NAME, NULL);
+    widget_set_font (view, monospace_font_name);
+    g_free (monospace_font_name);
+}
+
+gchar *
+prefs_get_monospace (void)
+{
+    return (gconf_client_get_string (client, GCONF_MONOSPACE_FONT_NAME, NULL));
+}
+
+gboolean
+prefs_get_have_tearoff (void)
+{
+    return (gconf_client_get_bool (client, GCONF_MENUS_HAVE_TEAROFF, NULL));
+}
+
+GSList *
+prefs_get_logs (void)
+{
+    return prefs->logs;
+}
+
+gchar *
+prefs_get_active_log (void) 
+{
+    return prefs->logfile;
+}
+
+int
+prefs_get_width (void)
+{
+    return prefs->width;
+}
+
+int
+prefs_get_height (void)
+{
+    return prefs->height;
 }
 
 void
-prefs_save (GConfClient *client, UserPrefsStruct *prefs)
+prefs_free_loglist ()
+{
+	g_slist_free (prefs->logs);
+	prefs->logs = NULL;
+}
+
+void
+prefs_store_log (gchar *name)
+{
+    if (name == NULL)
+        return;
+    prefs->logs = g_slist_append (prefs->logs, name);
+}
+
+void
+prefs_store_active_log (gchar *name)
+{
+    /* name can be NULL if no active log */
+    prefs->logfile = name;
+}
+
+void
+prefs_store_fontsize (int fontsize)
+{
+    prefs->fontsize = fontsize;
+}
+
+void
+prefs_save (void)
 {
   GSList *logs;
+
 	if (gconf_client_key_is_writable (client, GCONF_LOGFILE, NULL) &&
 	    prefs->logfile != NULL) {
 		gconf_client_set_string (client, GCONF_LOGFILE, prefs->logfile, NULL);
@@ -177,9 +258,12 @@ prefs_save (GConfClient *client, UserPrefsStruct *prefs)
 }
 
 void
-prefs_store_size (GtkWidget *window, UserPrefsStruct *prefs)
+prefs_store_window_size (GtkWidget *window)
 {
 	int width, height;
+
+    g_return_if_fail (GTK_IS_WINDOW (window));
+
 	gtk_window_get_size (GTK_WINDOW(window), &width, &height);
 	/* FIXME : we should check the state of the window, maximized or not */
 	if (width > 0 && height > 0) {
@@ -188,3 +272,20 @@ prefs_store_size (GtkWidget *window, UserPrefsStruct *prefs)
 	}
 }
 
+void 
+prefs_connect (LogviewWindow *logview)
+{
+    g_return_if_fail (LOGVIEW_IS_WINDOW (logview));
+
+    gconf_client_notify_add (client, GCONF_MONOSPACE_FONT_NAME, 
+                             prefs_monospace_font_changed, logview->view, NULL, NULL);
+}
+
+void
+prefs_init (int argc, char *argv[])
+{
+    gconf_init (argc, argv, NULL);
+    client = gconf_client_get_default ();
+    prefs = prefs_load (client);
+}
+
